## Copyright (C) 2018 jdonnal
## 
## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

## -*- texinfo -*- 
## @deftypefn {} {@var{retval} =} stat_plots (@var{input1}, @var{input2})
##
## @seealso{}
## @end deftypefn

## Author: jdonnal <jdonnal@officeserver>
## Created: 2018-10-25

d = load('joule_stats.dat');
d2 = load('joule_stats_repeat_inserter.dat');
hold off
plot(d(:,2))
hold on
plot(d2(:,2))
